@@ -577,6 +577,11 @@ static void HandleKeysSpecial(Key key)
     {
       DumpTape(&tape);
     }
+    else if (is_string_suffix(cheat_input, ":save-native-level") ||
+	     is_string_suffix(cheat_input, ":snl"))
+    {
+      SaveNativeLevel(&level);
+    }
   }
   else if (game_status == GAME_MODE_PLAYING)
   {
@@ -602,20 +607,22 @@ static void HandleKeysSpecial(Key key)
 void HandleKey(Key key, int key_status)
 {
   boolean anyTextGadgetActiveOrJustFinished = anyTextGadgetActive();
-  static struct SetupKeyboardInfo custom_key;
+  static struct SetupKeyboardInfo ski;
+  static struct SetupShortcutInfo ssi;
   static struct
   {
     Key *key_custom;
+    Key *key_snap;
     Key key_default;
     byte action;
   } key_info[] =
   {
-    { &custom_key.left,  DEFAULT_KEY_LEFT,  JOY_LEFT     },
-    { &custom_key.right, DEFAULT_KEY_RIGHT, JOY_RIGHT    },
-    { &custom_key.up,    DEFAULT_KEY_UP,    JOY_UP       },
-    { &custom_key.down,  DEFAULT_KEY_DOWN,  JOY_DOWN     },
-    { &custom_key.snap,  DEFAULT_KEY_SNAP,  JOY_BUTTON_1 },
-    { &custom_key.drop,  DEFAULT_KEY_DROP,  JOY_BUTTON_2 }
+    { &ski.left,  &ssi.snap_left,  DEFAULT_KEY_LEFT,  JOY_LEFT        },
+    { &ski.right, &ssi.snap_right, DEFAULT_KEY_RIGHT, JOY_RIGHT       },
+    { &ski.up,    &ssi.snap_up,    DEFAULT_KEY_UP,    JOY_UP          },
+    { &ski.down,  &ssi.snap_down,  DEFAULT_KEY_DOWN,  JOY_DOWN        },
+    { &ski.snap,  NULL,            DEFAULT_KEY_SNAP,  JOY_BUTTON_SNAP },
+    { &ski.drop,  NULL,            DEFAULT_KEY_DROP,  JOY_BUTTON_DROP }
   };
   int joy = 0;
   int i;
@@ -627,6 +634,8 @@ void HandleKey(Key key, int key_status)
     static boolean element_dropped[MAX_PLAYERS] = { FALSE,FALSE,FALSE,FALSE };
     int pnr;
 
+    ssi = setup.shortcut;
+
     for (pnr = 0; pnr < MAX_PLAYERS; pnr++)
     {
       byte key_action = 0;
@@ -634,12 +643,16 @@ void HandleKey(Key key, int key_status)
       if (setup.input[pnr].use_joystick)
 	continue;
 
-      custom_key = setup.input[pnr].key;
+      ski = setup.input[pnr].key;
 
-      for (i = 0; i < 6; i++)
+      for (i = 0; i < NUM_PLAYER_ACTIONS; i++)
 	if (key == *key_info[i].key_custom)
 	  key_action |= key_info[i].action;
 
+      for (i = 0; i < NUM_DIRECTIONS; i++)
+	if (key == *key_info[i].key_snap)
+	  key_action |= key_info[i].action | JOY_BUTTON_SNAP;
+
       if (tape.single_step && clear_button_2[pnr])
       {
 	stored_player[pnr].action &= ~KEY_BUTTON_2;
@@ -694,7 +707,7 @@ void HandleKey(Key key, int key_status)
   }
   else
   {
-    for (i = 0; i < 6; i++)
+    for (i = 0; i < NUM_PLAYER_ACTIONS; i++)
       if (key == key_info[i].key_default)
 	joy |= key_info[i].action;
   }
@@ -757,6 +770,9 @@ void HandleKey(Key key, int key_status)
       TapeQuickLoad();
     else if (key == setup.shortcut.toggle_pause)
       TapeTogglePause(TAPE_TOGGLE_MANUAL);
+
+    HandleTapeButtonKeys(key);
+    HandleSoundButtonKeys(key);
   }
 
   if (game_status == GAME_MODE_PLAYING && !network_playing)
@@ -863,6 +879,12 @@ void HandleKey(Key key, int key_status)
 	case KSYM_0:
 	  GameFrameDelay = (GameFrameDelay == 500 ? GAME_FRAME_DELAY : 500);
 	  break;
+
+	case KSYM_b:
+	  setup.sp_show_border_elements = !setup.sp_show_border_elements;
+	  printf("Supaplex border elements %s\n",
+		 setup.sp_show_border_elements ? "enabled" : "disabled");
+	  break;
 #endif
 
 	default:
