@@ -14,7 +14,7 @@ extern int debug;
 extern char *progname;
 extern char *arg_basedir;
 
-extern unsigned int frame;
+extern int frame;
 
 extern short ulaw_to_linear[256];
 extern unsigned char linear_to_ulaw[65536];
@@ -24,7 +24,11 @@ extern unsigned char linear_to_ulaw[65536];
 int open_all(void);
 void close_all(void);
 
+#if 1
+void readjoy(byte, struct PLAYER *);
+#else
 void readjoy(byte);
+#endif
 void input_eventloop(void);
 
 void blitscreen(void);
@@ -47,7 +51,7 @@ void synchro_1(void);
 void synchro_2(void);
 void synchro_3(void);
 
-int  cleanup_em_level(unsigned char *, int);
+int  cleanup_em_level(unsigned char *, int, char *);
 void convert_em_level(unsigned char *, int);
 void prepare_em_level(void);
 
