@@ -58,6 +58,14 @@ typedef unsigned char byte;
 #define SIGN(a)		((a) < 0 ? -1 : ((a) > 0 ? 1 : 0))
 #endif
 
+#ifndef ODD
+#define ODD(a)		(((a) & 1) == 1)
+#endif
+
+#ifndef EVEN
+#define EVEN(a)		(((a) & 1) == 0)
+#endif
+
 #define SIZEOF_ARRAY(array, type)	(sizeof(array) / sizeof(type))
 #define SIZEOF_ARRAY_INT(array)		SIZEOF_ARRAY(array, int)
 
