@@ -135,9 +135,9 @@ void DrawLevelElementShifted(int, int, int, int, int, int);
 void DrawScreenElementThruMask(int, int, int);
 void DrawLevelElementThruMask(int, int, int);
 void DrawLevelFieldThruMask(int, int);
-void DrawLevelFieldCrumbledSand(int, int);
-void DrawLevelFieldCrumbledSandDigging(int, int, int, int);
-void DrawLevelFieldCrumbledSandNeighbours(int, int);
+void DrawLevelFieldCrumbled(int, int);
+void DrawLevelFieldCrumbledDigging(int, int, int, int);
+void DrawLevelFieldCrumbledNeighbours(int, int);
 void DrawScreenElement(int, int, int);
 void DrawLevelElement(int, int, int);
 void DrawScreenField(int, int);
@@ -146,7 +146,8 @@ void DrawLevelField(int, int);
 void DrawMiniElement(int, int, int);
 void DrawMiniElementOrWall(int, int, int, int);
 
-void ShowEnvelope();
+void ShowEnvelope(int);
+void ShowEnvelopeDoor(char *text, int);
 
 void DrawLevel(void);
 void DrawMiniLevel(int, int, int, int);
@@ -174,6 +175,10 @@ int map_direction_EM_to_RND(int);
 void map_android_clone_elements_RND_to_EM(struct LevelInfo *);
 void map_android_clone_elements_EM_to_RND(struct LevelInfo *);
 
+int map_element_RND_to_SP(int);
+int map_element_SP_to_RND(int);
+int map_action_SP_to_RND(int);
+
 int get_next_element(int);
 int el_act_dir2img(int, int, int);
 int el_act2img(int, int);
@@ -211,5 +216,6 @@ void PlaySoundActivating();
 void PlaySoundSelecting();
 
 void ToggleFullscreenIfNeeded();
+void ChangeViewportPropertiesIfNeeded();
 
 #endif	/* TOOLS_H */
